@@ -67,7 +67,7 @@ static void soup_cache_session_feature_init (SoupSessionFeatureInterface *featur
 static SoupContentProcessorInterface *soup_cache_default_content_processor_interface;
 static void soup_cache_content_processor_init (SoupContentProcessorInterface *interface, gpointer interface_data);
 
-#define DEFAULT_MAX_SIZE 50 * 1024 * 1024
+#define DEFAULT_MAX_SIZE (50 * 1024 * 1024)
 #define MAX_ENTRY_DATA_PERCENTAGE 10 /* Percentage of the total size
 	                                of the cache that can be
 	                                filled by a single entry */
@@ -419,9 +419,9 @@ soup_cache_entry_set_freshness (SoupCacheEntry *entry, SoupMessage *msg, SoupCac
 		gint64 expires_t, date_t;
 
 		expires_d = soup_date_time_new_from_http_string (expires);
-		if (expires_d) {
-			date_d = soup_date_time_new_from_http_string (date);
+		date_d = soup_date_time_new_from_http_string (date);
 
+		if (expires_d && date_d) {
 			expires_t = g_date_time_to_unix (expires_d);
 			date_t = g_date_time_to_unix (date_d);
 
@@ -463,13 +463,15 @@ soup_cache_entry_set_freshness (SoupCacheEntry *entry, SoupMessage *msg, SoupCac
 		gint64 now, last_modified_t;
 
 		soup_date = soup_date_time_new_from_http_string (last_modified);
-		last_modified_t = g_date_time_to_unix (soup_date);
-		now = time (NULL);
+		if (soup_date) {
+			last_modified_t = g_date_time_to_unix (soup_date);
+			now = time (NULL);
 
 #define HEURISTIC_FACTOR 0.1 /* From Section 2.3.1.1 */
 
-		entry->freshness_lifetime = MAX (0, (now - last_modified_t) * HEURISTIC_FACTOR);
-		g_date_time_unref (soup_date);
+			entry->freshness_lifetime = MAX (0, (now - last_modified_t) * HEURISTIC_FACTOR);
+			g_date_time_unref (soup_date);
+		}
 	}
 
 	return;
@@ -484,6 +486,7 @@ soup_cache_entry_new (SoupCache *cache, SoupMessage *msg, time_t request_time, t
 {
 	SoupCacheEntry *entry;
 	const char *date;
+	GDateTime *soup_date = NULL;
 
 	entry = g_slice_new0 (SoupCacheEntry);
 	entry->dirty = FALSE;
@@ -504,13 +507,13 @@ soup_cache_entry_new (SoupCache *cache, SoupMessage *msg, time_t request_time, t
 
 	/* Section 2.3.2, Calculating Age */
 	date = soup_message_headers_get_one_common (entry->headers, SOUP_HEADER_DATE);
+	if (date)
+		soup_date = soup_date_time_new_from_http_string (date);
 
-	if (date) {
-		GDateTime *soup_date;
+	if (soup_date) {
 		const char *age;
 		gint64 date_value, apparent_age, corrected_received_age, response_delay, age_value = 0;
 
-		soup_date = soup_date_time_new_from_http_string (date);
 		date_value = g_date_time_to_unix (soup_date);
 		g_date_time_unref (soup_date);
 
@@ -1295,7 +1298,7 @@ soup_cache_has_response (SoupCache *cache, SoupMessage *msg)
  *
  * Calculates whether the @msg can be cached or not.
  *
- * Returns: a #SoupCacheability value indicating whether the @msg can be cached
+ * Returns: a [flags@Cacheability] value indicating whether the @msg can be cached
  *   or not.
  */
 SoupCacheability
@@ -1406,7 +1409,7 @@ clear_cache_files (SoupCache *cache)
  *
  * Will remove all entries in the @cache plus all the cache files.
  *
- * This is not thread safe and must be called only from the thread that created the #SoupCache
+ * This is not thread safe and must be called only from the thread that created the [class@Cache]
  */
 void
 soup_cache_clear (SoupCache *cache)
@@ -1471,11 +1474,11 @@ soup_cache_generate_conditional_request (SoupCache *cache, SoupMessage *original
 	if (last_modified)
 		soup_message_headers_append_common (soup_message_get_request_headers (msg),
                                                     SOUP_HEADER_IF_MODIFIED_SINCE,
-                                                    last_modified);
+                                                    last_modified, SOUP_HEADER_VALUE_TRUSTED);
 	if (etag)
 		soup_message_headers_append_common (soup_message_get_request_headers (msg),
                                                     SOUP_HEADER_IF_NONE_MATCH,
-                                                    etag);
+                                                    etag, SOUP_HEADER_VALUE_TRUSTED);
 
 	return msg;
 }
@@ -1568,7 +1571,7 @@ pack_entry (gpointer data,
  * You must call this before exiting if you want your cache data to
  * persist between sessions.
  *
- * This is not thread safe and must be called only from the thread that created the #SoupCache
+ * This is not thread safe and must be called only from the thread that created the [class@Cache]
  */
 void
 soup_cache_dump (SoupCache *cache)
@@ -1631,7 +1634,7 @@ insert_cache_file (SoupCache *cache, const char *name, GHashTable *leaked_entrie
  *
  * Loads the contents of @cache's index into memory.
  *
- * This is not thread safe and must be called only from the thread that created the #SoupCache
+ * This is not thread safe and must be called only from the thread that created the [class@Cache]
  */
 void
 soup_cache_load (SoupCache *cache)
