@@ -25,23 +25,23 @@
  *
  * Handles decoding of HTTP messages.
  *
- * #SoupContentDecoder handles adding the "Accept-Encoding" header on
+ * [class@ContentDecoder] handles adding the "Accept-Encoding" header on
  * outgoing messages, and processing the "Content-Encoding" header on
  * incoming ones. Currently it supports the "gzip", "deflate", and "br"
  * content codings.
  *
- * A #SoupContentDecoder will automatically be
+ * A [class@ContentDecoder] will automatically be
  * added to the session by default. (You can use
  * [method@Session.remove_feature_by_type] if you don't
  * want this.)
  *
- * If #SoupContentDecoder successfully decodes the Content-Encoding,
+ * If [class@ContentDecoder] successfully decodes the Content-Encoding,
  * the message body will contain the decoded data; however, the message headers
  * will be unchanged (and so "Content-Encoding" will still be present,
  * "Content-Length" will describe the original encoded length, etc).
  *
  * If "Content-Encoding" contains any encoding types that
- * #SoupContentDecoder doesn't recognize, then none of the encodings
+ * [class@ContentDecoder] doesn't recognize, then none of the encodings
  * will be decoded.
  *
  * (Note that currently there is no way to (automatically) use
@@ -249,7 +249,8 @@ soup_content_decoder_request_queued (SoupSessionFeature *feature,
 #endif
 
 		soup_message_headers_append_common (soup_message_get_request_headers (msg),
-                                                    SOUP_HEADER_ACCEPT_ENCODING, header);
+                                                    SOUP_HEADER_ACCEPT_ENCODING, header,
+                                                    SOUP_HEADER_VALUE_TRUSTED);
 	}
 }
 
