@@ -62,6 +62,7 @@ typedef struct {
         gboolean advertise_http2;
         SoupHTTPVersion http_version;
         SoupServerMessageIO *io_data;
+        GCancellable *cancellable;
 
         GSocketAddress *local_addr;
         GSocketAddress *remote_addr;
@@ -86,6 +87,7 @@ soup_server_connection_init (SoupServerConnection *conn)
         SoupServerConnectionPrivate *priv = soup_server_connection_get_instance_private (conn);
 
         priv->http_version = SOUP_HTTP_1_1;
+        priv->cancellable = g_cancellable_new ();
 }
 
 static void
@@ -98,6 +100,7 @@ disconnect_internal (SoupServerConnection *conn)
         g_io_stream_close (priv->conn, NULL, NULL);
         g_signal_handlers_disconnect_by_data (priv->conn, conn);
         g_clear_object (&priv->conn);
+        g_clear_object (&priv->initial_msg);
 
         g_clear_pointer (&priv->io_data, soup_server_message_io_destroy);
 }
@@ -108,6 +111,9 @@ soup_server_connection_finalize (GObject *object)
         SoupServerConnection *conn = SOUP_SERVER_CONNECTION (object);
         SoupServerConnectionPrivate *priv = soup_server_connection_get_instance_private (conn);
 
+        g_cancellable_cancel (priv->cancellable);
+        g_clear_object (&priv->cancellable);
+
         if (priv->conn) {
                 disconnect_internal (conn);
         } else {
@@ -427,8 +433,9 @@ tls_connection_handshake_ready_cb (GTlsConnection       *tls_conn,
                                    SoupServerConnection *conn)
 {
         SoupServerConnectionPrivate *priv = soup_server_connection_get_instance_private (conn);
+        GError *error = NULL;
 
-        if (g_tls_connection_handshake_finish (tls_conn, result, NULL)) {
+        if (g_tls_connection_handshake_finish (tls_conn, result, &error)) {
                 const char *protocol = g_tls_connection_get_negotiated_protocol (tls_conn);
 
                 if (g_strcmp0 (protocol, "h2") == 0)
@@ -439,7 +446,7 @@ tls_connection_handshake_ready_cb (GTlsConnection       *tls_conn,
                         priv->http_version = SOUP_HTTP_1_1;
 
                 soup_server_connection_connected (conn);
-        } else {
+        } else if (!g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
                 soup_server_connection_disconnect (conn);
         }
 }
@@ -517,7 +524,7 @@ soup_server_connection_accepted (SoupServerConnection *conn)
                                          conn, G_CONNECT_SWAPPED);
 
                 g_tls_connection_handshake_async (G_TLS_CONNECTION (priv->conn),
-                                                  G_PRIORITY_DEFAULT, NULL,
+                                                  G_PRIORITY_DEFAULT, priv->cancellable,
                                                   (GAsyncReadyCallback)tls_connection_handshake_ready_cb,
                                                   conn);
                 return;
