@@ -10,12 +10,18 @@
 
 G_BEGIN_DECLS
 
-void        soup_message_headers_append_untrusted_data  (SoupMessageHeaders *hdrs,
+typedef enum {
+        SOUP_HEADER_VALUE_UNTRUSTED,
+        SOUP_HEADER_VALUE_TRUSTED
+} SoupHeaderValueTrusted;
+
+gboolean    soup_message_headers_append_untrusted_data  (SoupMessageHeaders *hdrs,
                                                          const char         *name,
                                                          const char         *value);
-void        soup_message_headers_append_common          (SoupMessageHeaders *hdrs,
+gboolean    soup_message_headers_append_common          (SoupMessageHeaders *hdrs,
                                                          SoupHeaderName      name,
-                                                         const char         *value);
+                                                         const char         *value,
+                                                         SoupHeaderValueTrusted trusted_value);
 const char *soup_message_headers_get_one_common         (SoupMessageHeaders *hdrs,
                                                          SoupHeaderName      name);
 const char *soup_message_headers_get_list_common        (SoupMessageHeaders *hdrs,
@@ -24,7 +30,8 @@ void        soup_message_headers_remove_common          (SoupMessageHeaders *hdr
                                                          SoupHeaderName      name);
 void        soup_message_headers_replace_common         (SoupMessageHeaders *hdrs,
                                                          SoupHeaderName      name,
-                                                         const char         *value);
+                                                         const char         *value,
+                                                         SoupHeaderValueTrusted trusted_value);
 gboolean    soup_message_headers_header_contains_common (SoupMessageHeaders *hdrs,
                                                          SoupHeaderName      name,
                                                          const char         *token);
