@@ -10,6 +10,8 @@
 
 G_BEGIN_DECLS
 
+gboolean     soup_uri_is_valid              (GUri       *uri);
+
 gboolean     soup_uri_is_http               (GUri       *uri);
 
 gboolean     soup_uri_is_https              (GUri       *uri);
@@ -28,6 +30,4 @@ GUri        *soup_uri_copy_with_normalized_flags (GUri  *uri);
 
 char        *soup_uri_get_host_for_headers  (GUri       *uri);
 
-#define SOUP_URI_IS_VALID(x) ((x) && g_uri_get_host(x) && g_uri_get_host(x)[0])
-
 G_END_DECLS
