@@ -34,7 +34,7 @@
  *
  * A WebSocket connection.
  *
- * A #SoupWebsocketConnection is a WebSocket connection to a peer.
+ * A [class@WebsocketConnection] is a WebSocket connection to a peer.
  * This API is modeled after the W3C API for interacting with
  * WebSockets.
  *
@@ -925,6 +925,8 @@ process_contents (SoupWebsocketConnection *self,
 		}
 	} else if (priv->close_received) {
 		g_debug ("received message after close was received");
+        } else if (priv->close_sent && priv->dirty_close) {
+                g_debug ("received message after close due to error was sent");
 	} else {
 		/* A message frame */
 
@@ -1123,6 +1125,12 @@ process_frame (SoupWebsocketConnection *self)
 		payload += 4;
 		at += 4;
 
+		/* at has a maximum value of 10 + 4 = 14 */
+		if (payload_len > G_MAXSIZE - 14) {
+			bad_data_error_and_close (self);
+			return FALSE;
+		}
+
 		if (len < at + payload_len)
 			return FALSE; /* need more data */
 
@@ -1807,7 +1815,7 @@ soup_websocket_connection_class_init (SoupWebsocketConnectionClass *klass)
  * @protocol: (nullable): the subprotocol in use
  * @extensions: (element-type SoupWebsocketExtension) (transfer full): a #GList of #SoupWebsocketExtension objects
  *
- * Creates a #SoupWebsocketConnection on @stream with the given active @extensions.
+ * Creates a [class@WebsocketConnection] on @stream with the given active @extensions.
  *
  * This should be called after completing the handshake to begin using the WebSocket
  * protocol.
