@@ -6,6 +6,15 @@ typedef struct {
 	const char *name, *value;
 } Header;
 
+/* These are not C strings to ensure going one byte over is not safe. */
+static char unterminated_http_version[] = {
+        'G','E','T',' ','/',' ','H','T','T','P','/','1', '0', '0', '.'
+};
+
+static char only_newlines[] = {
+        '\n', '\n', '\n', '\n'
+};
+
 static struct RequestTest {
 	const char *description;
 	const char *bugref;
@@ -15,6 +24,7 @@ static struct RequestTest {
 	const char *method, *path;
 	SoupHTTPVersion version;
 	Header headers[10];
+	GLogLevelFlags log_flags;
 } reqtests[] = {
 	/**********************/
 	/*** VALID REQUESTS ***/
@@ -24,7 +34,7 @@ static struct RequestTest {
 	  "GET / HTTP/1.0\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "/", SOUP_HTTP_1_0,
-	  { { NULL } }
+	  { { NULL } }, 0
 	},
 
 	{ "Req w/ 1 header", NULL,
@@ -33,7 +43,7 @@ static struct RequestTest {
 	  "GET", "/", SOUP_HTTP_1_1,
 	  { { "Host", "example.com" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "Req w/ 1 header, no leading whitespace", NULL,
@@ -42,7 +52,7 @@ static struct RequestTest {
 	  "GET", "/", SOUP_HTTP_1_1,
 	  { { "Host", "example.com" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "Req w/ 1 header including trailing whitespace", NULL,
@@ -51,7 +61,7 @@ static struct RequestTest {
 	  "GET", "/", SOUP_HTTP_1_1,
 	  { { "Host", "example.com" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "Req w/ 1 header, wrapped", NULL,
@@ -60,7 +70,7 @@ static struct RequestTest {
 	  "GET", "/", SOUP_HTTP_1_1,
 	  { { "Foo", "bar baz" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "Req w/ 1 header, wrapped with additional whitespace", NULL,
@@ -69,7 +79,7 @@ static struct RequestTest {
 	  "GET", "/", SOUP_HTTP_1_1,
 	  { { "Foo", "bar baz" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "Req w/ 1 header, wrapped with tab", NULL,
@@ -78,7 +88,7 @@ static struct RequestTest {
 	  "GET", "/", SOUP_HTTP_1_1,
 	  { { "Foo", "bar baz" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "Req w/ 1 header, wrapped before value", NULL,
@@ -87,7 +97,7 @@ static struct RequestTest {
 	  "GET", "/", SOUP_HTTP_1_1,
 	  { { "Foo", "bar baz" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "Req w/ 1 header with empty value", NULL,
@@ -96,7 +106,7 @@ static struct RequestTest {
 	  "GET", "/", SOUP_HTTP_1_1,
 	  { { "Host", "" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "Req w/ 2 headers", NULL,
@@ -106,7 +116,7 @@ static struct RequestTest {
 	  { { "Host", "example.com" },
 	    { "Connection", "close" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "Req w/ 3 headers", NULL,
@@ -117,7 +127,7 @@ static struct RequestTest {
 	    { "Connection", "close" },
 	    { "Blah", "blah" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "Req w/ 3 headers, 1st wrapped", NULL,
@@ -128,7 +138,7 @@ static struct RequestTest {
             { "Foo", "bar baz" },
 	    { "Blah", "blah" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "Req w/ 3 headers, 2nd wrapped", NULL,
@@ -139,7 +149,7 @@ static struct RequestTest {
 	    { "Blah", "blah" },
 	    { "Foo", "bar baz" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "Req w/ 3 headers, 3rd wrapped", NULL,
@@ -150,7 +160,7 @@ static struct RequestTest {
 	    { "Blah", "blah" },
 	    { "Foo", "bar baz" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "Req w/ same header multiple times", NULL,
@@ -159,7 +169,7 @@ static struct RequestTest {
 	  "GET", "/", SOUP_HTTP_1_1,
 	  { { "Foo", "bar, baz, quux" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "Connection header on HTTP/1.0 message", NULL,
@@ -169,21 +179,21 @@ static struct RequestTest {
 	  { { "Connection", "Bar, Quux" },
             { "Foo", "bar" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "GET with full URI", "667637",
 	  "GET http://example.com HTTP/1.1\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "http://example.com", SOUP_HTTP_1_1,
-	  { { NULL } }
+	  { { NULL } }, 0
 	},
 
 	{ "GET with full URI in upper-case", "667637",
 	  "GET HTTP://example.com HTTP/1.1\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "HTTP://example.com", SOUP_HTTP_1_1,
-	  { { NULL } }
+	  { { NULL } }, 0
 	},
 
 	/* It's better for this to be passed through: this means a SoupServer
@@ -193,7 +203,7 @@ static struct RequestTest {
 	  "GET AbOuT: HTTP/1.1\r\n", -1,
 	  SOUP_STATUS_OK,
 	  "GET", "AbOuT:", SOUP_HTTP_1_1,
-	  { { NULL } }
+	  { { NULL } }, 0
 	},
 
 	/****************************/
@@ -208,7 +218,7 @@ static struct RequestTest {
 	  "GET", "/", SOUP_HTTP_1_1,
 	  { { "Host", "example.com" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	/* RFC 2616 section 3.1 says we MUST accept this */
@@ -219,7 +229,7 @@ static struct RequestTest {
 	  "GET", "/", SOUP_HTTP_1_1,
 	  { { "Host", "example.com" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	/* RFC 2616 section 19.3 says we SHOULD accept these */
@@ -231,7 +241,7 @@ static struct RequestTest {
 	  { { "Host", "example.com" },
 	    { "Connection", "close" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "LF instead of CRLF after Request-Line", NULL,
@@ -240,7 +250,7 @@ static struct RequestTest {
 	  "GET", "/", SOUP_HTTP_1_1,
 	  { { "Host", "example.com" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "Mixed CRLF/LF", "666316",
@@ -252,7 +262,7 @@ static struct RequestTest {
 	    { "e", "f" },
 	    { "g", "h" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "Req w/ incorrect whitespace in Request-Line", NULL,
@@ -261,7 +271,7 @@ static struct RequestTest {
 	  "GET", "/", SOUP_HTTP_1_1,
 	  { { "Host", "example.com" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "Req w/ incorrect whitespace after Request-Line", "475169",
@@ -270,7 +280,7 @@ static struct RequestTest {
 	  "GET", "/", SOUP_HTTP_1_1,
 	  { { "Host", "example.com" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	/* If the request/status line is parseable, then we
@@ -284,7 +294,7 @@ static struct RequestTest {
 	  { { "Host", "example.com" },
 	    { "Bar", "two" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "First header line is continuation", "666316",
@@ -294,7 +304,7 @@ static struct RequestTest {
 	  { { "Host", "example.com" },
 	    { "c", "d" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "Zero-length header name", "666316",
@@ -304,7 +314,7 @@ static struct RequestTest {
 	  { { "a", "b" },
 	    { "c", "d" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "CR in header name", "666316",
@@ -314,7 +324,7 @@ static struct RequestTest {
 	  { { "a", "b" },
 	    { "c", "d" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "CR in header value", "666316",
@@ -327,7 +337,7 @@ static struct RequestTest {
 	    { "s", "t" },		/* CR at end is ignored */
 	    { "c", "d" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "Tab in header name", "666316",
@@ -342,7 +352,7 @@ static struct RequestTest {
 	    { "p", "q z: w" },
 	    { "c", "d" },
 	    { NULL }
-	  }
+	  }, 0
 	},
 
 	{ "Tab in header value", "666316",
@@ -355,25 +365,7 @@ static struct RequestTest {
 	    { "z", "w" },	/* trailing tab ignored */
 	    { "c", "d" },
 	    { NULL }
-	  }
-	},
-
-	{ "NUL in header name", "760832",
-	  "GET / HTTP/1.1\r\nHost\x00: example.com\r\n", 36,
-	  SOUP_STATUS_OK,
-	  "GET", "/", SOUP_HTTP_1_1,
-	  { { "Host", "example.com" },
-	    { NULL }
-	  }
-	},
-
-	{ "NUL in header value", "760832",
-	  "GET / HTTP/1.1\r\nHost: example\x00" "com\r\n", 35,
-	  SOUP_STATUS_OK,
-	  "GET", "/", SOUP_HTTP_1_1,
-	  { { "Host", "examplecom" },
-	    { NULL }
-	  }
+	  }, 0
 	},
 
 	/************************/
@@ -384,70 +376,137 @@ static struct RequestTest {
 	  "GET /\r\n", -1,
 	  SOUP_STATUS_BAD_REQUEST,
 	  NULL, NULL, -1,
-	  { { NULL } }
+	  { { NULL } }, 0
 	},
 
 	{ "HTTP 1.2 request (no such thing)", NULL,
 	  "GET / HTTP/1.2\r\n", -1,
 	  SOUP_STATUS_HTTP_VERSION_NOT_SUPPORTED,
 	  NULL, NULL, -1,
-	  { { NULL } }
+	  { { NULL } }, 0
 	},
 
 	{ "HTTP 2000 request (no such thing)", NULL,
 	  "GET / HTTP/2000.0\r\n", -1,
 	  SOUP_STATUS_HTTP_VERSION_NOT_SUPPORTED,
 	  NULL, NULL, -1,
-	  { { NULL } }
+	  { { NULL } }, 0
+	},
+
+	{ "Long HTTP version terminating at missing minor version", "https://gitlab.gnome.org/GNOME/libsoup/-/issues/404",
+	  unterminated_http_version, sizeof (unterminated_http_version),
+	  SOUP_STATUS_BAD_REQUEST,
+           NULL, NULL, -1,
+	  { { NULL } }, 0
 	},
 
 	{ "Non-HTTP request", NULL,
 	  "GET / SOUP/1.1\r\nHost: example.com\r\n", -1,
 	  SOUP_STATUS_BAD_REQUEST,
 	  NULL, NULL, -1,
-	  { { NULL } }
+	  { { NULL } }, 0
 	},
 
 	{ "Junk after Request-Line", NULL,
 	  "GET / HTTP/1.1 blah\r\nHost: example.com\r\n", -1,
 	  SOUP_STATUS_BAD_REQUEST,
 	  NULL, NULL, -1,
-	  { { NULL } }
+	  { { NULL } }, 0
 	},
 
 	{ "NUL in Method", NULL,
 	  "G\x00T / HTTP/1.1\r\nHost: example.com\r\n", 37,
 	  SOUP_STATUS_BAD_REQUEST,
 	  NULL, NULL, -1,
-	  { { NULL } }
+	  { { NULL } }, 0
 	},
 
 	{ "NUL at beginning of Method", "666316",
 	  "\x00 / HTTP/1.1\r\nHost: example.com\r\n", 35,
 	  SOUP_STATUS_BAD_REQUEST,
 	  NULL, NULL, -1,
-	  { { NULL } }
+	  { { NULL } }, 0
 	},
 
 	{ "NUL in Path", NULL,
 	  "GET /\x00 HTTP/1.1\r\nHost: example.com\r\n", 38,
 	  SOUP_STATUS_BAD_REQUEST,
 	  NULL, NULL, -1,
-	  { { NULL } }
+	  { { NULL } }, 0
 	},
 
 	{ "No terminating CRLF", NULL,
 	  "GET / HTTP/1.1\r\nHost: example.com", -1,
 	  SOUP_STATUS_BAD_REQUEST,
 	  NULL, NULL, -1,
-	  { { NULL } }
+	  { { NULL } }, 0
 	},
 
 	{ "Unrecognized expectation", NULL,
 	  "GET / HTTP/1.1\r\nHost: example.com\r\nExpect: the-impossible\r\n", -1,
 	  SOUP_STATUS_EXPECTATION_FAILED,
 	  NULL, NULL, -1,
-	  { { NULL } }
+	  { { NULL } }, 0
+	},
+
+	// https://gitlab.gnome.org/GNOME/libsoup/-/issues/377
+	{ "NUL in header name", NULL,
+	  "GET / HTTP/1.1\r\nHost\x00: example.com\r\n", 36,
+	  SOUP_STATUS_BAD_REQUEST,
+	  NULL, NULL, -1,
+	  { { NULL } }, 0
+	},
+
+	{ "NUL in header value", NULL,
+	  "HTTP/1.1 200 OK\r\nFoo: b\x00" "ar\r\n", 28,
+	  SOUP_STATUS_BAD_REQUEST,
+           NULL, NULL, -1,
+	  { { NULL } }, 0
+	},
+
+	{ "CR and LF were incorrectly permitted within request URIs",
+	  "https://gitlab.gnome.org/GNOME/libsoup/-/issues/380",
+	  "GET /\r\n HTTP/1.1\r\nHost: example.com\r\n",
+	  -1,
+	  SOUP_STATUS_BAD_REQUEST,
+	  NULL, NULL, -1,
+	  { { NULL } }, 0
+	},
+
+	{ "CR and LF incorrectly allowed in request method",
+	  "https://gitlab.gnome.org/GNOME/libsoup/-/issues/441",
+	  "G\r\nET / HTTP/1.1\r\nHost: example.com\r\n",
+	  -1,
+	  SOUP_STATUS_BAD_REQUEST,
+	  NULL, NULL, -1,
+	  { { NULL } }, 0
+	},
+
+	{ "Only newlines", NULL,
+	  only_newlines, sizeof (only_newlines),
+	  SOUP_STATUS_BAD_REQUEST,
+           NULL, NULL, -1,
+	  { { NULL } }, 0
+	},
+
+	{ "Duplicate Host headers",
+	  "https://gitlab.gnome.org/GNOME/libsoup/-/issues/472",
+	  "GET / HTTP/1.1\r\nHost: example.com\r\nHost: example.org\r\n",
+	  -1,
+	  SOUP_STATUS_BAD_REQUEST,
+	  NULL, NULL, -1,
+	  { { NULL } },
+	  G_LOG_LEVEL_WARNING
+	},
+
+	{ "Duplicate Host headers, case insensitive",
+	  "https://gitlab.gnome.org/GNOME/libsoup/-/issues/472",
+	  "GET / HTTP/1.1\r\nHost: example.com\r\nhost: example.org\r\n",
+	  -1,
+	  SOUP_STATUS_BAD_REQUEST,
+	  NULL, NULL, -1,
+	  { { NULL } },
+	  G_LOG_LEVEL_WARNING
 	}
 };
 static const int num_reqtests = G_N_ELEMENTS (reqtests);
@@ -620,22 +679,6 @@ static struct ResponseTest {
 	    { NULL } }
 	},
 
-	{ "NUL in header name", "760832",
-	  "HTTP/1.1 200 OK\r\nF\x00oo: bar\r\n", 28,
-	  SOUP_HTTP_1_1, SOUP_STATUS_OK, "OK",
-	  { { "Foo", "bar" },
-	    { NULL }
-	  }
-	},
-
-	{ "NUL in header value", "760832",
-	  "HTTP/1.1 200 OK\r\nFoo: b\x00" "ar\r\n", 28,
-	  SOUP_HTTP_1_1, SOUP_STATUS_OK, "OK",
-	  { { "Foo", "bar" },
-	    { NULL }
-	  }
-	},
-
 	/********************************/
 	/*** VALID CONTINUE RESPONSES ***/
 	/********************************/
@@ -768,6 +811,19 @@ static struct ResponseTest {
 	  { { NULL }
 	  }
 	},
+
+	// https://gitlab.gnome.org/GNOME/libsoup/-/issues/377
+	{ "NUL in header name", NULL,
+	  "HTTP/1.1 200 OK\r\nF\x00oo: bar\r\n", 28,
+	  -1, 0, NULL,
+	  { { NULL } }
+	},
+
+	{ "NUL in header value", "760832",
+	  "HTTP/1.1 200 OK\r\nFoo: b\x00" "ar\r\n", 28,
+	  -1, 0, NULL,
+	  { { NULL } }
+	},
 };
 static const int num_resptests = G_N_ELEMENTS (resptests);
 
@@ -831,6 +887,17 @@ static struct ParamListTest {
 	    { "filename", "t\xC3\xA9st.txt" },
 	  },
 	},
+
+        /* This tests invalid UTF-8 data which *should* never be passed here but it was designed to be robust against it. */
+        { TRUE,
+              "invalid*=\x69\x27\x27\x93\x93\x93\x93\xff\x61\x61\x61\x61\x61\x61\x61\x62\x63\x64\x65\x0a; filename*=iso-8859-1''\x69\x27\x27\x93\x93\x93\x93\xff\x61\x61\x61\x61\x61\x61\x61\x62\x63\x64\x65\x0a; foo",
+              {
+                    { "filename", "i''\302\223\302\223\302\223\302\223\303\277aaaaaaabcde" },
+                    { "invalid", "\302\223\302\223\302\223\302\223\303\277aaaaaaabcde" },
+                    { "foo", NULL },
+
+                },
+        }
 };
 static const int num_paramlisttests = G_N_ELEMENTS (paramlisttests);
 
@@ -851,7 +918,7 @@ check_headers (Header *headers, SoupMessageHeaders *hdrs)
 	}
 
 	for (i = 0, h = header_names; headers[i].name && h; i++, h = h->next) {
-		g_assert (g_ascii_strcasecmp (h->data, headers[i].name) == 0);
+		g_assert_true (g_ascii_strcasecmp (h->data, headers[i].name) == 0);
 
 		value = soup_message_headers_get_list (hdrs, headers[i].name);
 		g_assert_cmpstr (value, ==, headers[i].value);
@@ -887,10 +954,17 @@ do_request_tests (void)
 			len = strlen (reqtests[i].request);
 		else
 			len = reqtests[i].length;
+
+		if (reqtests[i].log_flags)
+			g_test_expect_message ("libsoup", reqtests[i].log_flags, "*");
+
 		status = soup_headers_parse_request (reqtests[i].request, len,
 						     headers, &method, &path,
 						     &version);
 		g_assert_cmpint (status, ==, reqtests[i].status);
+		if (reqtests[i].log_flags)
+			g_test_assert_expected_messages ();
+
 		if (SOUP_STATUS_IS_SUCCESSFUL (status)) {
 			g_assert_cmpstr (method, ==, reqtests[i].method);
 			g_assert_cmpstr (path, ==, reqtests[i].path);
@@ -1034,6 +1108,18 @@ do_param_list_tests (void)
 #define RFC5987_TEST_HEADER_FALLBACK "attachment; filename*=Unknown''t%FF%FF%FFst.txt; filename=\"test.txt\""
 #define RFC5987_TEST_HEADER_NO_TYPE  "filename=\"test.txt\""
 #define RFC5987_TEST_HEADER_NO_TYPE_2  "filename=\"test.txt\"; foo=bar"
+#define RFC5987_TEST_HEADER_EMPTY_FILENAME ";filename"
+
+static const struct {
+        const char *description;
+        const char *filename;
+        const char *sanitized;
+} bad_filenames[] = {
+        { "Invalid leading", "/test.txt", "test.txt" },
+        { "Invalid trailing", "test.txt.\n", "test.txt" },
+        { "Invalid leading and trailing", " \ttest.txt/", "test.txt" },
+        { "Invalid characters", "**t/e:<s>t~|.t\n\rxt", "__t_e__s_t__.t__xt" }
+};
 
 static void
 do_content_disposition_tests (void)
@@ -1044,6 +1130,7 @@ do_content_disposition_tests (void)
 	char *disposition;
 	GBytes *buffer;
 	SoupMultipart *multipart;
+        int i;
 
 	hdrs = soup_message_headers_new (SOUP_MESSAGE_HEADERS_MULTIPART);
 	params = g_hash_table_new (g_str_hash, g_str_equal);
@@ -1134,6 +1221,38 @@ do_content_disposition_tests (void)
         g_assert_cmpstr (parameter2, ==, "bar");
 	g_hash_table_destroy (params);
 
+        /* Empty filename */
+        soup_message_headers_clear (hdrs);
+        soup_message_headers_append (hdrs, "Content-Disposition",
+				     RFC5987_TEST_HEADER_EMPTY_FILENAME);
+	if (!soup_message_headers_get_content_disposition (hdrs,
+							   &disposition,
+							   &params)) {
+		soup_test_assert (FALSE, "empty filename decoding FAILED");
+		return;
+	}
+        g_free (disposition);
+        g_assert_false (g_hash_table_contains (params, "filename"));
+	g_hash_table_destroy (params);
+
+        /* Invalid filenames */
+        for (i = 0; i < G_N_ELEMENTS (bad_filenames); i++) {
+                debug_printf (1, "  %s \n", bad_filenames[i].description);
+
+                soup_message_headers_clear (hdrs);
+                params = g_hash_table_new (g_str_hash, g_str_equal);
+                g_hash_table_insert (params, "filename", (char*)bad_filenames[i].filename);
+                soup_message_headers_set_content_disposition (hdrs, "attachment", params);
+                g_hash_table_destroy (params);
+
+                g_assert_true (soup_message_headers_get_content_disposition (hdrs, &disposition, &params));
+                g_free (disposition);
+
+                filename = g_hash_table_lookup (params, "filename");
+                g_assert_cmpstr (filename, ==, bad_filenames[i].sanitized);
+                g_hash_table_destroy (params);
+        }
+
 	soup_message_headers_unref (hdrs);
 
 	/* Ensure that soup-multipart always quotes filename */
@@ -1240,16 +1359,21 @@ do_append_param_tests (void)
 
 static const struct {
 	const char *description, *name, *value;
-} bad_headers[] = {
-	{ "Empty name", "", "value" },
-	{ "Name with spaces", "na me", "value" },
-	{ "Name with colon", "na:me", "value" },
-	{ "Name with CR", "na\rme", "value" },
-	{ "Name with LF", "na\nme", "value" },
-	{ "Name with tab", "na\tme", "value" },
-	{ "Value with CR", "name", "val\rue" },
-	{ "Value with LF", "name", "val\nue" },
-	{ "Value with LWS", "name", "val\r\n ue" }
+} bad_header_names[] = {
+	{ "empty name", "", "value" },
+	{ "name with spaces", "na me", "value" },
+	{ "name with colon", "na:me", "value" },
+	{ "name with CR", "na\rme", "value" },
+	{ "name with LF", "na\nme", "value" },
+	{ "name with tab", "na\tme", "value" }
+};
+
+static const struct {
+        const char *description, *name, *value;
+} bad_header_values[] = {
+	{ "value with CR", "name", "val\rue" },
+	{ "value with LF", "name", "val\nue" },
+	{ "value with LWS", "name", "val\r\n ue" }
 };
 
 static void
@@ -1259,15 +1383,95 @@ do_bad_header_tests (void)
 	int i;
 
 	hdrs = soup_message_headers_new (SOUP_MESSAGE_HEADERS_MULTIPART);
-	for (i = 0; i < G_N_ELEMENTS (bad_headers); i++) {
-		debug_printf (1, "  %s\n", bad_headers[i].description);
 
-		g_test_expect_message ("libsoup", G_LOG_LEVEL_CRITICAL,
-				       "*soup_message_headers_append*assertion*failed*");
-		soup_message_headers_append (hdrs, bad_headers[i].name,
-					     bad_headers[i].value);
-		g_test_assert_expected_messages ();
+        /* soup_message_headers_append: bad names */
+	for (i = 0; i < G_N_ELEMENTS (bad_header_names); i++) {
+		debug_printf (1, "  Append %s\n", bad_header_names[i].description);
+
+                g_test_expect_message ("libsoup", G_LOG_LEVEL_WARNING,
+                                       "*soup_message_headers_append*Rejecting bad name*");
+		soup_message_headers_append (hdrs, bad_header_names[i].name,
+					     bad_header_names[i].value);
+                g_test_assert_expected_messages ();
 	}
+
+        /* soup_message_headers_append: bad values */
+        for (i = 0; i < G_N_ELEMENTS (bad_header_values); i++) {
+		debug_printf (1, "  Append %s\n", bad_header_values[i].description);
+
+                g_test_expect_message ("libsoup", G_LOG_LEVEL_WARNING,
+                                       "*soup_message_headers_append*Rejecting bad value*");
+		soup_message_headers_append (hdrs, bad_header_values[i].name,
+					     bad_header_values[i].value);
+                g_test_assert_expected_messages ();
+	}
+
+        /* soup_message_headers_replace: bad values */
+        for (i = 0; i < G_N_ELEMENTS (bad_header_values); i++) {
+		debug_printf (1, "  Replace %s\n", bad_header_values[i].description);
+
+                g_test_expect_message ("libsoup", G_LOG_LEVEL_WARNING,
+                                       "*soup_message_headers_append*Rejecting bad value*");
+		soup_message_headers_replace (hdrs, bad_header_values[i].name,
+                                              bad_header_values[i].value);
+                g_test_assert_expected_messages ();
+	}
+
+        /* soup_message_headers_set_content_type: bad values */
+        for (i = 0; i < G_N_ELEMENTS (bad_header_values); i++) {
+                GHashTable *params;
+
+                debug_printf (1, "  Content type with %s\n", bad_header_values[i].description);
+
+                g_test_expect_message ("libsoup", G_LOG_LEVEL_WARNING,
+                                       "*soup_message_headers_append*Rejecting bad value*");
+                soup_message_headers_set_content_type (hdrs, bad_header_values[i].value, NULL);
+                g_test_assert_expected_messages ();
+
+                g_test_expect_message ("libsoup", G_LOG_LEVEL_WARNING,
+                                       "*soup_message_headers_append*Rejecting bad value*");
+                params = g_hash_table_new (g_str_hash, g_str_equal);
+                g_hash_table_insert (params, CONTENT_TYPE_TEST_ATTRIBUTE, (gpointer)bad_header_values[i].value);
+                soup_message_headers_set_content_type (hdrs, CONTENT_TYPE_TEST_MIME_TYPE, params);
+                g_hash_table_destroy (params);
+                g_test_assert_expected_messages ();
+        }
+
+        /* soup_message_headers_set_content_disposition: bad values */
+        for (i = 0; i < G_N_ELEMENTS (bad_header_values); i++) {
+                debug_printf (1, "  Content disposition with %s\n", bad_header_values[i].description);
+
+                g_test_expect_message ("libsoup", G_LOG_LEVEL_WARNING,
+                                       "*soup_message_headers_append*Rejecting bad value*");
+                soup_message_headers_set_content_disposition (hdrs, bad_header_values[i].value, NULL);
+                g_test_assert_expected_messages ();
+        }
+	soup_message_headers_unref (hdrs);
+}
+
+static void
+do_append_duplicate_host_test (void)
+{
+	SoupMessageHeaders *hdrs;
+	const char *list_value;
+
+	hdrs = soup_message_headers_new (SOUP_MESSAGE_HEADERS_REQUEST);
+	soup_message_headers_append (hdrs, "Host", "a");
+
+	g_test_expect_message ("libsoup", G_LOG_LEVEL_WARNING,
+		               "soup_message_headers_append_common: Rejecting duplicate Host header");
+	soup_message_headers_append (hdrs, "Host", "b");
+	g_test_assert_expected_messages ();
+
+        /* Case insensitive */
+	g_test_expect_message ("libsoup", G_LOG_LEVEL_WARNING,
+		               "soup_message_headers_append_common: Rejecting duplicate Host header");
+	soup_message_headers_append (hdrs, "host", "b");
+	g_test_assert_expected_messages ();
+
+	list_value = soup_message_headers_get_list (hdrs, "Host");
+	g_assert_cmpstr (list_value, ==, "a");
+
 	soup_message_headers_unref (hdrs);
 }
 
@@ -1286,6 +1490,7 @@ main (int argc, char **argv)
 	g_test_add_func ("/header-parsing/content-type", do_content_type_tests);
 	g_test_add_func ("/header-parsing/append-param", do_append_param_tests);
 	g_test_add_func ("/header-parsing/bad", do_bad_header_tests);
+	g_test_add_func ("/header-parsing/append-duplicate-host", do_append_duplicate_host_test);
 
 	ret = g_test_run ();
 
