@@ -69,7 +69,7 @@ on_error_copy (SoupWebsocketConnection *ws,
                gpointer user_data)
 {
 	GError **copy = user_data;
-	g_assert (*copy == NULL);
+	g_assert_null (*copy);
 	*copy = g_error_copy (error);
 }
 
@@ -151,7 +151,7 @@ got_connection (GSocket *listener,
 	g_assert_no_error (error);
 
 	conn = g_socket_connection_factory_create_connection (sock);
-	g_assert (conn != NULL);
+	g_assert_nonnull (conn);
 	g_object_unref (sock);
 
 	if (test->no_server)
@@ -360,8 +360,8 @@ on_text_message (SoupWebsocketConnection *ws,
 	GBytes **receive = user_data;
 
 	g_assert_cmpint (type, ==, SOUP_WEBSOCKET_DATA_TEXT);
-	g_assert (*receive == NULL);
-	g_assert (message != NULL);
+	g_assert_null (*receive);
+	g_assert_nonnull (message);
 
 	*receive = g_bytes_ref (message);
 }
@@ -375,8 +375,8 @@ on_binary_message (SoupWebsocketConnection *ws,
 	GBytes **receive = user_data;
 
 	g_assert_cmpint (type, ==, SOUP_WEBSOCKET_DATA_BINARY);
-	g_assert (*receive == NULL);
-	g_assert (message != NULL);
+	g_assert_null (*receive);
+	g_assert_nonnull (message);
 
 	*receive = g_bytes_ref (message);
 }
@@ -400,7 +400,7 @@ on_close_set_flag (SoupWebsocketConnection *ws,
 {
 	gboolean *flag = user_data;
 
-	g_assert (*flag == FALSE);
+	g_assert_false (*flag);
 
 	*flag = TRUE;
 }
@@ -416,7 +416,7 @@ test_handshake (Test *test,
 
 		g_assert_nonnull (extensions);
 		g_assert_cmpuint (g_list_length (extensions), ==, 1);
-		g_assert (SOUP_IS_WEBSOCKET_EXTENSION_DEFLATE (extensions->data));
+		g_assert_true (SOUP_IS_WEBSOCKET_EXTENSION_DEFLATE (extensions->data));
 	} else {
 		g_assert_null (soup_websocket_connection_get_extensions (test->client));
 	}
@@ -427,7 +427,7 @@ test_handshake (Test *test,
 
                 g_assert_nonnull (extensions);
                 g_assert_cmpuint (g_list_length (extensions), ==, 1);
-                g_assert (SOUP_IS_WEBSOCKET_EXTENSION_DEFLATE (extensions->data));
+                g_assert_true (SOUP_IS_WEBSOCKET_EXTENSION_DEFLATE (extensions->data));
         } else {
 		g_assert_null (soup_websocket_connection_get_extensions (test->server));
 	}
@@ -515,7 +515,7 @@ test_send_client_to_server (Test *test,
 
 	WAIT_UNTIL (received != NULL);
 
-	g_assert (g_bytes_equal (sent, received));
+	g_assert_true (g_bytes_equal (sent, received));
 	g_clear_pointer (&sent, g_bytes_unref);
 	g_clear_pointer (&received, g_bytes_unref);
 }
@@ -580,7 +580,7 @@ test_send_server_to_client (Test *test,
 
         WAIT_UNTIL (received != NULL);
 
-        g_assert (g_bytes_equal (sent, received));
+        g_assert_true (g_bytes_equal (sent, received));
         g_clear_pointer (&sent, g_bytes_unref);
         g_clear_pointer (&received, g_bytes_unref);
 }
@@ -597,7 +597,7 @@ test_send_big_packets (Test *test,
 	sent = g_bytes_new_take (g_strnfill (400, '!'), 400);
 	soup_websocket_connection_send_text (test->server, g_bytes_get_data (sent, NULL));
 	WAIT_UNTIL (received != NULL);
-	g_assert (g_bytes_equal (sent, received));
+	g_assert_true (g_bytes_equal (sent, received));
 	g_bytes_unref (sent);
 	g_bytes_unref (received);
 	received = NULL;
@@ -605,20 +605,20 @@ test_send_big_packets (Test *test,
 	sent = g_bytes_new_take (g_strnfill (100 * 1000, '?'), 100 * 1000);
 	soup_websocket_connection_send_text (test->server, g_bytes_get_data (sent, NULL));
 	WAIT_UNTIL (received != NULL);
-	g_assert (g_bytes_equal (sent, received));
+	g_assert_true (g_bytes_equal (sent, received));
 	g_bytes_unref (sent);
 	g_bytes_unref (received);
 	received = NULL;
 
 	soup_websocket_connection_set_max_incoming_payload_size (test->client, 1000 * 1000 + 1);
-	g_assert (soup_websocket_connection_get_max_incoming_payload_size (test->client) == (1000 * 1000 + 1));
+	g_assert_true (soup_websocket_connection_get_max_incoming_payload_size (test->client) == (1000 * 1000 + 1));
 	soup_websocket_connection_set_max_incoming_payload_size (test->server, 1000 * 1000 + 1);
-	g_assert (soup_websocket_connection_get_max_incoming_payload_size (test->server) == (1000 * 1000 + 1));
+	g_assert_true (soup_websocket_connection_get_max_incoming_payload_size (test->server) == (1000 * 1000 + 1));
 
 	sent = g_bytes_new_take (g_strnfill (1000 * 1000, '?'), 1000 * 1000);
 	soup_websocket_connection_send_text (test->server, g_bytes_get_data (sent, NULL));
 	WAIT_UNTIL (received != NULL);
-	g_assert (g_bytes_equal (sent, received));
+	g_assert_true (g_bytes_equal (sent, received));
 	g_bytes_unref (sent);
 	g_bytes_unref (received);
 }
@@ -674,7 +674,7 @@ test_send_bad_data (Test *test,
 	frame = "\x81\x84\x00\x00\x00\x00\xEE\xEE\xEE\xEE";
 	if (!g_output_stream_write_all (g_io_stream_get_output_stream (io),
 					frame, 10, &written, NULL, NULL))
-		g_assert_not_reached ();
+		g_assert_cmpstr ("This code", ==, "should not be reached");
 	g_assert_cmpuint (written, ==, 10);
 
 	WAIT_UNTIL (error != NULL);
@@ -682,7 +682,7 @@ test_send_bad_data (Test *test,
 	g_clear_error (&error);
 
 	WAIT_UNTIL (soup_websocket_connection_get_state (test->client) == SOUP_WEBSOCKET_STATE_CLOSED);
-	g_assert (close_event);
+	g_assert_true (close_event);
 
 	g_assert_cmpuint (soup_websocket_connection_get_close_code (test->client), ==, SOUP_WEBSOCKET_CLOSE_BAD_DATA);
 }
@@ -1052,8 +1052,8 @@ do_close_clean_client (Test *test,
 	WAIT_UNTIL (soup_websocket_connection_get_state (test->server) == SOUP_WEBSOCKET_STATE_CLOSED);
 	WAIT_UNTIL (soup_websocket_connection_get_state (test->client) == SOUP_WEBSOCKET_STATE_CLOSED);
 
-	g_assert (close_event_client);
-	g_assert (close_event_server);
+	g_assert_true (close_event_client);
+	g_assert_true (close_event_server);
 
 	g_assert_cmpint (soup_websocket_connection_get_close_code (test->client), ==, expected_sender_code);
 	g_assert_cmpstr (soup_websocket_connection_get_close_data (test->client), ==, expected_sender_reason);
@@ -1130,8 +1130,8 @@ do_close_clean_server (Test *test,
 	WAIT_UNTIL (soup_websocket_connection_get_state (test->server) == SOUP_WEBSOCKET_STATE_CLOSED);
 	WAIT_UNTIL (soup_websocket_connection_get_state (test->client) == SOUP_WEBSOCKET_STATE_CLOSED);
 
-	g_assert (close_event_client);
-	g_assert (close_event_server);
+	g_assert_true (close_event_client);
+	g_assert_true (close_event_server);
 
 	g_assert_cmpint (soup_websocket_connection_get_close_code (test->server), ==, expected_sender_code);
 	g_assert_cmpstr (soup_websocket_connection_get_close_data (test->server), ==, expected_sender_reason);
@@ -1219,11 +1219,11 @@ test_message_after_closing (Test *test,
 	WAIT_UNTIL (soup_websocket_connection_get_state (test->server) == SOUP_WEBSOCKET_STATE_CLOSED);
 	WAIT_UNTIL (soup_websocket_connection_get_state (test->client) == SOUP_WEBSOCKET_STATE_CLOSED);
 
-	g_assert (close_event_client);
-	g_assert (close_event_server);
+	g_assert_true (close_event_client);
+	g_assert_true (close_event_server);
 
-	g_assert (received != NULL);
-	g_assert (g_bytes_equal (message, received));
+	g_assert_true (received != NULL);
+	g_assert_true (g_bytes_equal (message, received));
 
 	g_bytes_unref (received);
 	g_bytes_unref (message);
@@ -1340,7 +1340,7 @@ test_close_after_timeout (Test *test,
 
 	WAIT_UNTIL (soup_websocket_connection_get_state (test->client) == SOUP_WEBSOCKET_STATE_CLOSED);
 
-	g_assert (close_event == TRUE);
+	g_assert_true (close_event);
 
 	/* Now actually close the server side stream */
 	g_mutex_unlock (&test->mutex);
@@ -1400,7 +1400,7 @@ send_compressed_fragments_server_thread (gpointer user_data)
         GError *error = NULL;
 
         memset (&zstream, 0, sizeof(z_stream));
-        g_assert (deflateInit2 (&zstream, Z_DEFAULT_COMPRESSION, Z_DEFLATED, -15, 8, Z_DEFAULT_STRATEGY) == Z_OK);
+        g_assert_true (deflateInit2 (&zstream, Z_DEFAULT_COMPRESSION, Z_DEFLATED, -15, 8, Z_DEFAULT_STRATEGY) == Z_OK);
 
         data = g_byte_array_new ();
 
@@ -1459,7 +1459,7 @@ test_receive_fragmented (Test *test,
 
 	WAIT_UNTIL (received != NULL);
 	expect = g_bytes_new ("one two three", 13);
-	g_assert (g_bytes_equal (expect, received));
+	g_assert_true (g_bytes_equal (expect, received));
 	g_bytes_unref (expect);
 	g_bytes_unref (received);
 
@@ -1508,8 +1508,9 @@ test_receive_invalid_encode_length_16 (Test *test,
 	GError *error = NULL;
 	InvalidEncodeLengthTest context = { test, NULL };
 	guint i;
+	guint error_id;
 
-	g_signal_connect (test->client, "error", G_CALLBACK (on_error_copy), &error);
+	error_id = g_signal_connect (test->client, "error", G_CALLBACK (on_error_copy), &error);
 	g_signal_connect (test->client, "message", G_CALLBACK (on_binary_message), &received);
 
 	/* We use 126(~) as payload length with 125 extended length */
@@ -1522,6 +1523,7 @@ test_receive_invalid_encode_length_16 (Test *test,
 	WAIT_UNTIL (error != NULL || received != NULL);
 	g_assert_error (error, SOUP_WEBSOCKET_ERROR, SOUP_WEBSOCKET_CLOSE_PROTOCOL_ERROR);
 	g_clear_error (&error);
+        g_signal_handler_disconnect (test->client, error_id);
 	g_assert_null (received);
 
 	g_thread_join (thread);
@@ -1539,8 +1541,9 @@ test_receive_invalid_encode_length_64 (Test *test,
 	GError *error = NULL;
 	InvalidEncodeLengthTest context = { test, NULL };
 	guint i;
+	guint error_id;
 
-	g_signal_connect (test->client, "error", G_CALLBACK (on_error_copy), &error);
+	error_id = g_signal_connect (test->client, "error", G_CALLBACK (on_error_copy), &error);
 	g_signal_connect (test->client, "message", G_CALLBACK (on_binary_message), &received);
 
 	/* We use 127(\x7f) as payload length with 65535 extended length */
@@ -1553,6 +1556,7 @@ test_receive_invalid_encode_length_64 (Test *test,
 	WAIT_UNTIL (error != NULL || received != NULL);
 	g_assert_error (error, SOUP_WEBSOCKET_ERROR, SOUP_WEBSOCKET_CLOSE_PROTOCOL_ERROR);
 	g_clear_error (&error);
+        g_signal_handler_disconnect (test->client, error_id);
 	g_assert_null (received);
 
         g_thread_join (thread);
@@ -1598,6 +1602,9 @@ test_client_receive_masked_frame (Test *test,
 	g_clear_error (&error);
 	g_assert_null (received);
 
+	/* it can emit more errors while joining the thread, thus disconnect, to avoid memory leak */
+	g_signal_handlers_disconnect_by_func (test->client, G_CALLBACK (on_error_copy), &error);
+
         g_thread_join (thread);
 
 	WAIT_UNTIL (soup_websocket_connection_get_state (test->client) == SOUP_WEBSOCKET_STATE_CLOSED);
@@ -1623,7 +1630,7 @@ test_server_receive_unmasked_frame (Test *test,
 	frame = "\x81\x0bHello World";
 	if (!g_output_stream_write_all (g_io_stream_get_output_stream (io),
 					frame, 13, &written, NULL, NULL))
-		g_assert_not_reached ();
+		g_assert_cmpstr ("This code", ==, "should not be reached");
 	g_assert_cmpuint (written, ==, 13);
 
 	WAIT_UNTIL (error != NULL);
@@ -1631,7 +1638,7 @@ test_server_receive_unmasked_frame (Test *test,
 	g_clear_error (&error);
 
 	WAIT_UNTIL (soup_websocket_connection_get_state (test->client) == SOUP_WEBSOCKET_STATE_CLOSED);
-	g_assert (close_event);
+	g_assert_true (close_event);
 
 	g_assert_cmpuint (soup_websocket_connection_get_close_code (test->client), ==, SOUP_WEBSOCKET_CLOSE_PROTOCOL_ERROR);
 
@@ -1845,7 +1852,7 @@ test_deflate_negotiate_direct (Test *test,
 								deflate_negotiate_tests[i].server_supports_extensions ?
 								supported_extensions : NULL,
 								&error);
-		g_assert (result == deflate_negotiate_tests[i].expected_check_result);
+		g_assert_true (result == deflate_negotiate_tests[i].expected_check_result);
 		if (result) {
 			g_assert_no_error (error);
 		} else {
@@ -1857,7 +1864,7 @@ test_deflate_negotiate_direct (Test *test,
 								  deflate_negotiate_tests[i].server_supports_extensions ?
 								  supported_extensions : NULL,
 								  &accepted_extensions);
-		g_assert (result == deflate_negotiate_tests[i].expected_check_result);
+		g_assert_true (result == deflate_negotiate_tests[i].expected_check_result);
 
 		soup_message_set_status (msg, soup_server_message_get_status (server_msg), NULL);
 		response_headers = soup_server_message_get_response_headers (server_msg);
@@ -1871,7 +1878,7 @@ test_deflate_negotiate_direct (Test *test,
 			g_assert_cmpstr (extension, ==, deflate_negotiate_tests[i].server_extension);
 			g_assert_nonnull (accepted_extensions);
 			g_assert_cmpuint (g_list_length (accepted_extensions), ==, 1);
-			g_assert (SOUP_IS_WEBSOCKET_EXTENSION_DEFLATE (accepted_extensions->data));
+			g_assert_true (SOUP_IS_WEBSOCKET_EXTENSION_DEFLATE (accepted_extensions->data));
 			g_list_free_full (accepted_extensions, g_object_unref);
 			accepted_extensions = NULL;
 		} else {
@@ -1879,7 +1886,7 @@ test_deflate_negotiate_direct (Test *test,
 		}
 
 		result = soup_websocket_client_verify_handshake (msg, supported_extensions, &accepted_extensions, &error);
-		g_assert (result == deflate_negotiate_tests[i].expected_verify_result);
+		g_assert_true (result == deflate_negotiate_tests[i].expected_verify_result);
 		if (result) {
                         g_assert_no_error (error);
                 } else {
@@ -1889,7 +1896,7 @@ test_deflate_negotiate_direct (Test *test,
 		if (deflate_negotiate_tests[i].expected_accepted_extension) {
 			g_assert_nonnull (accepted_extensions);
                         g_assert_cmpuint (g_list_length (accepted_extensions), ==, 1);
-                        g_assert (SOUP_IS_WEBSOCKET_EXTENSION_DEFLATE (accepted_extensions->data));
+                        g_assert_true (SOUP_IS_WEBSOCKET_EXTENSION_DEFLATE (accepted_extensions->data));
                         g_list_free_full (accepted_extensions, g_object_unref);
                         accepted_extensions = NULL;
                 } else {
@@ -1983,7 +1990,7 @@ send_compressed_fragments_error_server_thread (gpointer user_data)
         GError *error = NULL;
 
         memset (&zstream, 0, sizeof(z_stream));
-        g_assert (deflateInit2 (&zstream, Z_DEFAULT_COMPRESSION, Z_DEFLATED, -15, 8, Z_DEFAULT_STRATEGY) == Z_OK);
+        g_assert_true (deflateInit2 (&zstream, Z_DEFAULT_COMPRESSION, Z_DEFLATED, -15, 8, Z_DEFAULT_STRATEGY) == Z_OK);
 
         data = g_byte_array_new ();
 
@@ -2046,10 +2053,13 @@ test_deflate_receive_fragmented_error (Test *test,
 	g_clear_error (&error);
 	g_assert_null (received);
 
+	/* it can emit more errors while joining the thread, thus disconnect, to avoid memory leak */
+	g_signal_handlers_disconnect_by_func (test->client, G_CALLBACK (on_error_copy), &error);
+
 	g_thread_join (thread);
 
 	WAIT_UNTIL (soup_websocket_connection_get_state (test->client) == SOUP_WEBSOCKET_STATE_CLOSED);
-	g_assert (close_event);
+	g_assert_true (close_event);
 }
 
 static void
@@ -2142,6 +2152,69 @@ test_connection_error (void)
 	soup_test_session_abort_unref (session);
 }
 
+static void
+test_fragment_assembly_corruption (Test *test, gconstpointer data)
+{
+        GBytes *received = NULL;
+        gsize written;
+        const char fragments[] =
+                "\x01\x10START_FRAGMENT_1" /* valid */
+                "\x02\x10WRONG_OPCODE_XXX" /* Wrong opcode! */
+                "\x00\x08INJECTED"
+                "\x80\x05_DATA"; /* Final fragment */
+        GError *error = NULL;
+
+        g_signal_connect (test->client, "error", G_CALLBACK (on_error_copy), &error);
+        g_signal_connect (test->client, "message", G_CALLBACK (on_text_message), &received);
+
+        g_output_stream_write_all (g_io_stream_get_output_stream (test->raw_server),
+                                   fragments, sizeof (fragments) - 1, &written, NULL, &error);
+        g_assert_no_error (error);
+        g_assert_cmpuint (written, ==, sizeof (fragments) - 1);
+
+        WAIT_UNTIL (error != NULL || received != NULL);
+
+        g_assert_null (received);
+        g_assert_error (error, SOUP_WEBSOCKET_ERROR, SOUP_WEBSOCKET_CLOSE_PROTOCOL_ERROR);
+        g_clear_error (&error);
+        g_clear_pointer (&received, g_bytes_unref);
+}
+
+static void
+test_cve_2026_0716 (Test *test,
+                    gconstpointer unused)
+{
+	GError *error = NULL;
+	GIOStream *io;
+	gsize written;
+	const char *frame;
+	gboolean close_event = FALSE;
+
+	g_signal_handlers_disconnect_by_func (test->server, on_error_not_reached, NULL);
+	g_signal_connect (test->server, "error", G_CALLBACK (on_error_copy), &error);
+	g_signal_connect (test->client, "closed", G_CALLBACK (on_close_set_flag), &close_event);
+
+	io = soup_websocket_connection_get_io_stream (test->client);
+
+	soup_websocket_connection_set_max_incoming_payload_size (test->server, 0);
+
+	// Malicious masked frame header (10-byte header + 4-byte mask) */
+	frame = "\x82\xff\xff\xff\xff\xff\xff\xff\xff\xf6\xaa\xbb\xcc\xdd";
+	if (!g_output_stream_write_all (g_io_stream_get_output_stream (io),
+					frame, 14, &written, NULL, NULL))
+		g_assert_cmpstr ("This code", ==, "should not be reached");
+	g_assert_cmpuint (written, ==, 14);
+
+	WAIT_UNTIL (error != NULL);
+	g_assert_error (error, SOUP_WEBSOCKET_ERROR, SOUP_WEBSOCKET_CLOSE_BAD_DATA);
+	g_clear_error (&error);
+
+	WAIT_UNTIL (soup_websocket_connection_get_state (test->client) == SOUP_WEBSOCKET_STATE_CLOSED);
+	g_assert_true (close_event);
+
+	g_assert_cmpuint (soup_websocket_connection_get_close_code (test->client), ==, SOUP_WEBSOCKET_CLOSE_BAD_DATA);
+}
+
 int
 main (int argc,
       char *argv[])
@@ -2388,6 +2461,20 @@ main (int argc,
 
 	g_test_add_func ("/websocket/soup/connection-error", test_connection_error);
 
+        g_test_add ("/websocket/direct/fragment-assembly-corruption", Test, NULL,
+                    setup_half_direct_connection,
+                    test_fragment_assembly_corruption,
+                    teardown_direct_connection);
+
+	g_test_add ("/websocket/direct/cve-2026-0716", Test, NULL,
+		    setup_direct_connection,
+		    test_cve_2026_0716,
+		    teardown_direct_connection);
+	g_test_add ("/websocket/soup/cve-2026-0716", Test, NULL,
+		    setup_soup_connection,
+		    test_cve_2026_0716,
+		    teardown_soup_connection);
+
 	ret = g_test_run ();
 
 	test_cleanup ();
